package game.command;

import java.util.List;
import java.util.Set;

import game.collect.Lists;
import game.collect.Sets;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.init.EntityRegistry;
import game.network.Player;
import game.world.WorldServer;

public class EntityListParser extends EntityParser {
	public EntityListParser(String name, boolean useSender, boolean livingOnly) {
		super(name, useSender, livingOnly);
	}

	public Object parse(CommandEnvironment env, String input) {
		if(input.equals("**")) {
			List<Entity> list = Lists.newArrayList();
			for(WorldServer world : env.getServer().getWorlds()) {
				if(this.livingOnly) {
					for(Entity ent : world.getEntities()) {
						if(ent instanceof EntityLiving)
							list.add(ent);
					}
				}
				else {
					list.addAll(world.getEntities());
				}
			}
			if(list.isEmpty())
				throw new RunException(this.livingOnly ? "Keine lebendigen Objekte gefunden" : "Keine Objekte gefunden");
			return list;
		}
		else if(input.equals("*")) {
			List<Entity> list = Lists.newArrayList();
			for(Player plr : env.getServer().getPlayers()) {
				if(plr.getEntity() != null)
					list.add(plr.getEntity());
			}
			if(list.isEmpty())
				throw new RunException("Keine Spieler gefunden");
			return list;
		}
		Set<Entity> set = Sets.newHashSet();
		for(String tok : input.split(",", -1)) {
			Class<? extends Entity> clazz = EntityRegistry.getEntityClass(tok);
			if(clazz != null) {
				if(this.livingOnly && !EntityLiving.class.isAssignableFrom(clazz))
					throw new RunException("Objekttyp %s ist nicht lebendig", EntityRegistry.getEntityName(EntityRegistry.getEntityString(clazz)));
				for(WorldServer world : env.getServer().getWorlds()) {
					for(Entity ent : world.getEntities()) {
						if(clazz.isAssignableFrom(ent.getClass()))
							set.add(ent);
					}
				}
			}
			else {
				set.add((Entity)super.parse(env, tok));
			}
		}
		if(set.isEmpty())
			throw new RunException("Keine Objekte gefunden");
		return Lists.newArrayList(set);
	}
	
	public Object getDefault(CommandEnvironment env) {
		Entity entity = (Entity)super.getDefault(env);
		return entity == null ? null : Lists.newArrayList(entity);
	}
	
	public String[] getCompletions(CommandEnvironment env) {
		Entity target = env.getExecutor().getPointedEntity();
		List<String> comp = target == null || (this.livingOnly && !(target instanceof EntityLiving)) ? Lists.newArrayList() : Lists.newArrayList("#" + target.getId());
		comp.addAll(env.getServer().getAllUsernames());
		comp.add("*");
		for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
			if(!this.livingOnly || EntityLiving.class.isAssignableFrom(clazz))
				comp.add(EntityRegistry.getEntityString(clazz));
		}
		comp.add("**");
		return comp.toArray(new String[comp.size()]);
	}

	public Class<?> getTypeClass() {
		return List.class;
	}
}
