package game.command.commands;

import java.util.List;
import game.command.CommandEnvironment;
import game.command.Command;
import game.command.Executor;
import game.command.RunException;
import game.command.StringCompleter;
import game.entity.Entity;
import game.init.Config;
import game.init.UniverseRegistry;
import game.world.BlockPos;
import game.world.Position;
import game.world.WorldServer;

public class CommandWarp extends Command {
	public CommandWarp() {
		super("warp");
		
		this.addString("warp", true, new StringCompleter() {
			public String[] complete(CommandEnvironment env) {
				String[] warps = new String[1 + env.getServer().getWarps().size()];
				warps[0] = "spawn";
				int pos = 1;
				for(String warp : env.getServer().getWarps().keySet()) {
					warps[pos++] = warp;
				}
				return warps;
			}
		});
		
		this.addEntityList("entities", 'e', true);
	}
	
	public Object exec(CommandEnvironment env, Executor exec, String warp, List<Entity> entities) {
		Position pos;
		if(warp.equals("spawn")) {
			WorldServer world = env.getServer().getWorld(Config.spawnDim);
			world = world == null ? env.getServer().getSpace() : world;
			int y = Config.spawnY;
	    	while(world.getState(new BlockPos(Config.spawnX, y, Config.spawnZ)).getBlock().getMaterial().blocksMovement() && y < 511)
	            y++;
	    	pos = new Position(Config.spawnX + 0.5d, (double)y, Config.spawnZ + 0.5d, 
					Config.spawnYaw, Config.spawnPitch, world.dimension.getDimensionId());
		}
		else {
			pos = env.getServer().getWarps().get(warp);
			if(pos == null)
				throw new RunException("Warp '%s' existiert nicht", warp);
			else if(env.getServer().getWorld(pos.dim) == null)
				throw new RunException("Warp '%s' hat kein Level (%s)", warp, pos.dim);
		}
		for(Entity entity : entities) {
	    	entity.teleport(pos);
			exec.logConsole("%s nach %d, %d, %d in %s teleportiert", entity.getCommandName(), (int)pos.x, (int)pos.y, (int)pos.z, UniverseRegistry.getDimension(pos.dim).getFormattedName(false));
		}
		return entities.size();
	}
}
